'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDraggable = require('react-draggable');

var _reactResizable = require('react-resizable');

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * An individual item within a ReactGridLayout.
 */

var GridItem = function (_React$Component) {
  _inherits(GridItem, _React$Component);

  function GridItem() {
    var _Object$getPrototypeO;

    var _temp, _this, _ret;

    _classCallCheck(this, GridItem);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, (_Object$getPrototypeO = Object.getPrototypeOf(GridItem)).call.apply(_Object$getPrototypeO, [this].concat(args))), _this), _this.state = {
      resizing: null,
      dragging: null,
      className: ''
    }, _temp), _possibleConstructorReturn(_this, _ret);
  }

  _createClass(GridItem, [{
    key: 'calcPosition',


    /**
     * Return position on the page given an x, y, w, h.
     * left, top, width, height are all in pixels.
     * @param  {Number}  x             X coordinate in grid units.
     * @param  {Number}  y             Y coordinate in grid units.
     * @param  {Number}  w             W coordinate in grid units.
     * @param  {Number}  h             H coordinate in grid units.
     * @return {Object}                Object containing coords.
     */
    value: function calcPosition(x, y, w, h) {
      var _props = this.props;
      var margin = _props.margin;
      var containerWidth = _props.containerWidth;
      var cols = _props.cols;
      var rowHeight = _props.rowHeight;


      var width = containerWidth - margin[0];
      var out = {
        left: Math.round(width * (x / cols) + margin[0]),
        top: Math.round(rowHeight * y + margin[1]),
        width: Math.round(width * (w / cols) - margin[0]),
        height: Math.round(h * rowHeight - margin[1])
      };

      if (this.state.resizing) {
        out.width = this.state.resizing.width;
        out.height = this.state.resizing.height;
      }

      if (this.state.dragging) {
        out.top = this.state.dragging.top;
        out.left = this.state.dragging.left;
      }

      return out;
    }

    /**
     * Translate x and y coordinates from pixels to grid units.
     * @param  {Number} top  Top position (relative to parent) in pixels.
     * @param  {Number} left Left position (relative to parent) in pixels.
     * @return {Object} x and y in grid units.
     */

  }, {
    key: 'calcXY',
    value: function calcXY(top, left) {
      var _props2 = this.props;
      var margin = _props2.margin;
      var containerWidth = _props2.containerWidth;
      var cols = _props2.cols;
      var rowHeight = _props2.rowHeight;
      var w = _props2.w;


      left -= margin[0];
      top -= margin[1];
      // This is intentional; because so much of the logic on moving boxes up/down relies
      // on an exact y position, we only round the x, not the y.
      var x = Math.round(left / containerWidth * cols);
      var y = Math.floor(top / rowHeight);
      x = Math.max(Math.min(x, cols), 0);
      y = Math.max(y, 0);

      // Cap x at numCols
      x = Math.min(x, cols - w);

      return { x: x, y: y };
    }

    /**
     * Given a height and width in pixel values, calculate grid units.
     * @param  {Number} height Height in pixels.
     * @param  {Number} width  Width in pixels.
     * @return {Object} w, h as grid units.
     */

  }, {
    key: 'calcWH',
    value: function calcWH(_ref) {
      var height = _ref.height;
      var width = _ref.width;
      var _props3 = this.props;
      var margin = _props3.margin;
      var containerWidth = _props3.containerWidth;
      var cols = _props3.cols;
      var rowHeight = _props3.rowHeight;
      var x = _props3.x;


      width += margin[0];
      height += margin[1];
      var w = Math.round(width / containerWidth * cols);
      var h = Math.round(height / rowHeight);
      w = Math.max(Math.min(w, cols - x), 0);
      h = Math.max(h, 0);
      return { w: w, h: h };
    }

    /**
     * This is where we set the grid item's absolute placement. It gets a little tricky because we want to do it
     * well when server rendering, and the only way to do that properly is to use percentage width/left because
     * we don't know exactly what the browser viewport is.
     * Unfortunately, CSS Transforms, which are great for performance, break in this instance because a percentage
     * left is relative to the item itself, not its container! So we cannot use them on the server rendering pass.
     *
     * @param  {Object} pos Position object with width, height, left, top.
     * @return {Object}     Style object.
     */

  }, {
    key: 'createStyle',
    value: function createStyle(pos) {
      var _props4 = this.props;
      var usePercentages = _props4.usePercentages;
      var containerWidth = _props4.containerWidth;
      var useCSSTransforms = _props4.useCSSTransforms;


      var style = {
        width: pos.width + 'px',
        height: pos.height + 'px',
        left: pos.left + 'px',
        top: pos.top + 'px',
        position: 'absolute'
      };

      // This is used for server rendering.
      if (usePercentages) {
        style.left = (0, _utils.perc)(pos.left / containerWidth);
        style.width = (0, _utils.perc)(pos.width / containerWidth);
      }

      // CSS Transforms support
      if (useCSSTransforms) {
        (0, _utils.setTransform)(style, [pos.left, pos.top]);
        style.left = null;
        style.top = null;
      }

      return style;
    }

    /**
     * Mix a Draggable instance into a child.
     * @param  {Element} child    Child element.
     * @return {Element}          Child wrapped in Draggable.
     */

  }, {
    key: 'mixinDraggable',
    value: function mixinDraggable(child) {
      return _react2.default.createElement(
        _reactDraggable.DraggableCore,
        {
          onStart: this.onDragHandler('onDragStart'),
          onDrag: this.onDragHandler('onDrag'),
          onStop: this.onDragHandler('onDragStop'),
          handle: this.props.handle,
          cancel: ".react-resizable-handle " + this.props.cancel },
        child
      );
    }

    /**
     * Mix a Resizable instance into a child.
     * @param  {Element} child    Child element.
     * @param  {Object} position  Position object (pixel values)
     * @return {Element}          Child wrapped in Resizable.
     */

  }, {
    key: 'mixinResizable',
    value: function mixinResizable(child, position) {
      var _props5 = this.props;
      var cols = _props5.cols;
      var x = _props5.x;
      var minW = _props5.minW;
      var minH = _props5.minH;
      var maxW = _props5.maxW;
      var maxH = _props5.maxH;

      // This is the max possible width - doesn't go to infinity because of the width of the window

      var maxWidth = this.calcPosition(0, 0, cols - x, 0).width;

      // Calculate min/max constraints using our min & maxes
      var mins = this.calcPosition(0, 0, minW, minH);
      var maxes = this.calcPosition(0, 0, maxW, maxH);
      var minConstraints = [mins.width, mins.height];
      var maxConstraints = [Math.min(maxes.width, maxWidth), Math.min(maxes.height, Infinity)];
      return _react2.default.createElement(
        _reactResizable.Resizable,
        {
          width: position.width,
          height: position.height,
          minConstraints: minConstraints,
          maxConstraints: maxConstraints,
          onResizeStop: this.onResizeHandler('onResizeStop'),
          onResizeStart: this.onResizeHandler('onResizeStart'),
          onResize: this.onResizeHandler('onResize') },
        child
      );
    }

    /**
     * Wrapper around drag events to provide more useful data.
     * All drag events call the function with the given handler name,
     * with the signature (index, x, y).
     *
     * @param  {String} handlerName Handler name to wrap.
     * @return {Function}           Handler function.
     */

  }, {
    key: 'onDragHandler',
    value: function onDragHandler(handlerName) {
      var _this2 = this;

      return function (e, _ref2) {
        var node = _ref2.node;
        var position = _ref2.position;

        if (!_this2.props[handlerName]) return;

        var newPosition = { top: 0, left: 0 };

        // Get new XY
        switch (handlerName) {
          case 'onDragStart':
            // ToDo this wont work on nested parents
            var parentRect = node.offsetParent.getBoundingClientRect();
            var clientRect = node.getBoundingClientRect();
            newPosition.top = clientRect.top - parentRect.top;
            newPosition.left = clientRect.left - parentRect.left;
            _this2.setState({ dragging: newPosition });
            break;
          case 'onDrag':
            if (!_this2.state.dragging) throw new Error('onDrag called before onDragStart.');
            newPosition.left = _this2.state.dragging.left + position.deltaX;
            newPosition.top = _this2.state.dragging.top + position.deltaY;
            _this2.setState({ dragging: newPosition });
            break;
          case 'onDragStop':
            if (!_this2.state.dragging) throw new Error('onDragEnd called before onDragStart.');
            newPosition.left = _this2.state.dragging.left;
            newPosition.top = _this2.state.dragging.top;
            _this2.setState({ dragging: null });
            break;
          default:
            throw new Error('onDragHandler called with unrecognized handlerName: ' + handlerName);
        }

        var _calcXY = _this2.calcXY(newPosition.top, newPosition.left);

        var x = _calcXY.x;
        var y = _calcXY.y;


        _this2.props[handlerName](_this2.props.i, x, y, { e: e, node: node, newPosition: newPosition });
      };
    }

    /**
     * Wrapper around drag events to provide more useful data.
     * All drag events call the function with the given handler name,
     * with the signature (index, x, y).
     *
     * @param  {String} handlerName Handler name to wrap.
     * @return {Function}           Handler function.
     */

  }, {
    key: 'onResizeHandler',
    value: function onResizeHandler(handlerName) {
      var _this3 = this;

      return function (e, _ref3) {
        var element = _ref3.element;
        var size = _ref3.size;

        if (!_this3.props[handlerName]) return;
        var _props6 = _this3.props;
        var cols = _props6.cols;
        var x = _props6.x;
        var i = _props6.i;
        var maxW = _props6.maxW;
        var minW = _props6.minW;
        var maxH = _props6.maxH;
        var minH = _props6.minH;

        // Get new XY

        var _calcWH = _this3.calcWH(size);

        var w = _calcWH.w;
        var h = _calcWH.h;

        // Cap w at numCols

        w = Math.min(w, cols - x);
        // Ensure w is at least 1
        w = Math.max(w, 1);

        // Min/max capping
        w = Math.max(Math.min(w, maxW), minW);
        h = Math.max(Math.min(h, maxH), minH);

        _this3.setState({ resizing: handlerName === 'onResizeStop' ? null : size });

        _this3.props[handlerName](i, w, h, { e: e, element: element, size: size });
      };
    }
  }, {
    key: 'render',
    value: function render() {
      var _props7 = this.props;
      var x = _props7.x;
      var y = _props7.y;
      var w = _props7.w;
      var h = _props7.h;
      var className = _props7.className;
      var style = _props7.style;
      var isDraggable = _props7.isDraggable;
      var isPlaceholder = _props7.isPlaceholder;
      var isResizable = _props7.isResizable;
      var useCSSTransforms = _props7.useCSSTransforms;


      var pos = this.calcPosition(x, y, w, h);
      var child = _react2.default.Children.only(this.props.children);

      // Create the child element. We clone the existing element but modify its className and style.
      var newChild = _react2.default.cloneElement(child, {
        // Munge a classname. Use passed in classnames and resizing.
        // React with merge the classNames.
        className: ['react-grid-item', className, isDraggable || isPlaceholder ? '' : 'static', this.state.resizing ? 'resizing' : '', this.state.dragging ? 'react-draggable-dragging' : '', useCSSTransforms ? 'cssTransforms' : ''].join(' '),
        // We can set the width and height on the child, but unfortunately we can't set the position.
        style: _extends({ style: style }, this.createStyle(pos))
      });

      // Resizable support. This is usually on but the user can toggle it off.
      if (isResizable) newChild = this.mixinResizable(newChild, pos);

      // Draggable support. This is always on, except for with placeholders.
      if (isDraggable) newChild = this.mixinDraggable(newChild);

      return newChild;
    }
  }]);

  return GridItem;
}(_react2.default.Component);

GridItem.propTypes = {
  // Children must be only a single element
  children: _react.PropTypes.element,

  // General grid attributes
  cols: _react.PropTypes.number.isRequired,
  containerWidth: _react.PropTypes.number.isRequired,
  rowHeight: _react.PropTypes.number.isRequired,
  margin: _react.PropTypes.array.isRequired,

  // These are all in grid units
  x: _react.PropTypes.number.isRequired,
  y: _react.PropTypes.number.isRequired,
  w: _react.PropTypes.number.isRequired,
  h: _react.PropTypes.number.isRequired,

  // All optional
  minW: function minW(props, propName, componentName, location, propFullName) {
    _react.PropTypes.number(props, propName, componentName, location, propFullName);
    var value = props[propName];
    if (value > props.w || value > props.maxW) return new Error('minWidth bigger than item width/maxWidth');
  },
  maxW: function maxW(props, propName, componentName, location, propFullName) {
    _react.PropTypes.number(props, propName, componentName, location, propFullName);
    var value = props[propName];
    if (value < props.w || value < props.minW) return new Error('maxWidth smaller than item width/minWidth');
  },
  minH: function minH(props, propName, componentName, location, propFullName) {
    _react.PropTypes.number(props, propName, componentName, location, propFullName);
    var value = props[propName];
    if (value > props.h || value > props.maxH) return new Error('minHeight bigger than item height/maxHeight');
  },
  maxH: function maxH(props, propName, componentName, location, propFullName) {
    _react.PropTypes.number(props, propName, componentName, location, propFullName);
    var value = props[propName];
    if (value < props.h || value < props.minH) return new Error('maxHeight smaller than item height/minHeight');
  },

  // ID is nice to have for callbacks
  i: _react.PropTypes.string.isRequired,

  // Functions
  onDragStop: _react.PropTypes.func,
  onDragStart: _react.PropTypes.func,
  onDrag: _react.PropTypes.func,
  onResizeStop: _react.PropTypes.func,
  onResizeStart: _react.PropTypes.func,
  onResize: _react.PropTypes.func,

  // Flags
  isDraggable: _react.PropTypes.bool.isRequired,
  isResizable: _react.PropTypes.bool.isRequired,

  // Use CSS transforms instead of top/left
  useCSSTransforms: _react.PropTypes.bool.isRequired,
  isPlaceholder: _react.PropTypes.bool,

  // Others
  className: _react.PropTypes.string,
  // Selector for draggable handle
  handle: _react.PropTypes.string,
  // Selector for draggable cancel (see react-draggable)
  cancel: _react.PropTypes.string
};
GridItem.defaultProps = {
  isPlaceholder: false,
  className: '',
  cancel: '',
  minH: 1,
  minW: 1,
  maxH: Infinity,
  maxW: Infinity
};
exports.default = GridItem;