'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _lodash = require('lodash.isequal');

var _lodash2 = _interopRequireDefault(_lodash);

var _utils = require('./utils');

var _GridItem = require('./GridItem');

var _GridItem2 = _interopRequireDefault(_GridItem);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

// Types

var noop = function noop() {};
// End Types

/**
 * A reactive, fluid grid layout with draggable, resizable components.
 */

var ReactGridLayout = function (_React$Component) {
  _inherits(ReactGridLayout, _React$Component);

  // TODO publish internal ReactClass displayName transform

  function ReactGridLayout(props, context) {
    _classCallCheck(this, ReactGridLayout);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(ReactGridLayout).call(this, props, context));

    _initialiseProps.call(_this);

    (0, _utils.autoBindHandlers)(_this, ['onDragStart', 'onDrag', 'onDragStop', 'onResizeStart', 'onResize', 'onResizeStop']);
    return _this;
  }

  _createClass(ReactGridLayout, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      // Call back with layout on mount. This should be done after correcting the layout width
      // to ensure we don't rerender with the wrong width.
      this.props.onLayoutChange(this.state.layout);
      this.setState({ isMounted: true });
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      // Allow parent to set layout directly.
      if (!(0, _lodash2.default)(nextProps.layout, this.state.layout)) {
        this.setState({
          layout: (0, _utils.synchronizeLayoutWithChildren)(nextProps.layout, nextProps.children, nextProps.cols, nextProps.verticalCompact)
        });
        // Call back so we can store the layout
        // Do it only when a resize/drag is not active, otherwise there are way too many callbacks
        if (!this.state.activeDrag) this.props.onLayoutChange(this.state.layout);
      }
      // If children change, regenerate the layout.
      if (nextProps.children.length !== this.props.children.length) {
        this.setState({
          layout: (0, _utils.synchronizeLayoutWithChildren)(this.state.layout, nextProps.children, nextProps.cols, nextProps.verticalCompact)
        });
      }
    }

    /**
     * Calculates a pixel value for the container.
     * @return {String} Container height in pixels.
     */

  }, {
    key: 'containerHeight',
    value: function containerHeight() {
      if (!this.props.autoSize) return;
      return (0, _utils.bottom)(this.state.layout) * this.props.rowHeight + this.props.margin[1] + 'px';
    }

    /**
     * When dragging starts
     * @param {String} i Id of the child
     * @param {Number} x X position of the move
     * @param {Number} y Y position of the move
     * @param {Event} e The mousedown event
     * @param {Element} node The current dragging DOM element
     */

  }, {
    key: 'onDragStart',
    value: function onDragStart(i, x, y, _ref) {
      var e = _ref.e;
      var node = _ref.node;
      var layout = this.state.layout;

      var l = (0, _utils.getLayoutItem)(layout, i);
      if (!l) return;

      this.setState({ oldDragItem: (0, _utils.clone)(l) });

      this.props.onDragStart(layout, l, l, null, e, node);
    }

    /**
     * Each drag movement create a new dragelement and move the element to the dragged location
     * @param {String} i Id of the child
     * @param {Number} x X position of the move
     * @param {Number} y Y position of the move
     * @param {Event} e The mousedown event
     * @param {Element} node The current dragging DOM element
     */

  }, {
    key: 'onDrag',
    value: function onDrag(i, x, y, _ref2) {
      var e = _ref2.e;
      var node = _ref2.node;
      var oldDragItem = this.state.oldDragItem;
      var layout = this.state.layout;

      var l = (0, _utils.getLayoutItem)(layout, i);
      if (!l) return;

      // Create placeholder (display only)
      var placeholder = {
        w: l.w, h: l.h, x: l.x, y: l.y, placeholder: true, i: i
      };

      // Move the element to the dragged location.
      layout = (0, _utils.moveElement)(layout, l, x, y, true /* isUserAction */);

      this.props.onDrag(layout, oldDragItem, l, placeholder, e, node);

      this.setState({
        layout: (0, _utils.compact)(layout, this.props.verticalCompact),
        activeDrag: placeholder
      });
    }

    /**
     * When dragging stops, figure out which position the element is closest to and update its x and y.
     * @param  {String} i Index of the child.
     * @param {Number} x X position of the move
     * @param {Number} y Y position of the move
     * @param {Event} e The mousedown event
     * @param {Element} node The current dragging DOM element
     */

  }, {
    key: 'onDragStop',
    value: function onDragStop(i, x, y, _ref3) {
      var e = _ref3.e;
      var node = _ref3.node;
      var oldDragItem = this.state.oldDragItem;
      var layout = this.state.layout;

      var l = (0, _utils.getLayoutItem)(layout, i);
      if (!l) return;

      // Move the element here
      layout = (0, _utils.moveElement)(layout, l, x, y, true /* isUserAction */);

      this.props.onDragStop(layout, oldDragItem, l, null, e, node);

      // Set state
      this.setState({
        layout: (0, _utils.compact)(layout, this.props.verticalCompact),
        activeDrag: null,
        oldDragItem: null
      });
    }
  }, {
    key: 'onResizeStart',
    value: function onResizeStart(i, w, h, _ref4) {
      var e = _ref4.e;
      var node = _ref4.node;
      var layout = this.state.layout;

      var l = (0, _utils.getLayoutItem)(layout, i);
      if (!l) return;

      this.setState({ oldResizeItem: (0, _utils.clone)(l) });

      this.props.onResizeStart(layout, l, l, null, e, node);
    }
  }, {
    key: 'onResize',
    value: function onResize(i, w, h, _ref5) {
      var e = _ref5.e;
      var node = _ref5.node;
      var _state = this.state;
      var layout = _state.layout;
      var oldResizeItem = _state.oldResizeItem;

      var l = (0, _utils.getLayoutItem)(layout, i);
      if (!l) return;

      // Set new width and height.
      l.w = w;
      l.h = h;

      // Create placeholder element (display only)
      var placeholder = {
        w: w, h: h, x: l.x, y: l.y, placeholder: true, i: i
      };

      this.props.onResize(layout, oldResizeItem, l, placeholder, e, node);

      // Re-compact the layout and set the drag placeholder.
      this.setState({ layout: (0, _utils.compact)(layout, this.props.verticalCompact), activeDrag: placeholder });
    }
  }, {
    key: 'onResizeStop',
    value: function onResizeStop(i, w, h, _ref6) {
      var e = _ref6.e;
      var node = _ref6.node;
      var _state2 = this.state;
      var layout = _state2.layout;
      var oldResizeItem = _state2.oldResizeItem;

      var l = (0, _utils.getLayoutItem)(layout, i);

      this.props.onResizeStop(layout, oldResizeItem, l, null, e, node);

      this.setState({
        layout: (0, _utils.compact)(layout, this.props.verticalCompact),
        activeDrag: null,
        oldResizeItem: null
      });
    }

    /**
     * Create a placeholder object.
     * @return {Element} Placeholder div.
     */

  }, {
    key: 'placeholder',
    value: function placeholder() {
      var activeDrag = this.state.activeDrag;

      if (!activeDrag) return null;
      var _props = this.props;
      var width = _props.width;
      var offsetX = _props.offsetX;
      var offsetY = _props.offsetY;
      var cols = _props.cols;
      var margin = _props.margin;
      var rowHeight = _props.rowHeight;
      var useCSSTransforms = _props.useCSSTransforms;

      // {...this.state.activeDrag} is pretty slow, actually

      return _react2.default.createElement(
        _GridItem2.default,
        {
          w: activeDrag.w,
          h: activeDrag.h,
          x: activeDrag.x,
          y: activeDrag.y,
          i: activeDrag.i,
          isPlaceholder: true,
          className: 'react-grid-placeholder',
          containerWidth: width,
          offsetX: offsetX,
          offsetY: offsetY,
          cols: cols,
          margin: margin,
          rowHeight: rowHeight,
          isDraggable: false,
          isResizable: false,
          useCSSTransforms: useCSSTransforms },
        _react2.default.createElement('div', null)
      );
    }

    /**
     * Given a grid item, set its style attributes & surround in a <Draggable>.
     * @param  {Element} child React element.
     * @return {Element}       Element wrapped in draggable and properly placed.
     */

  }, {
    key: 'processGridItem',
    value: function processGridItem(child) {
      if (!child.key) return;
      var l = (0, _utils.getLayoutItem)(this.state.layout, child.key);
      if (!l) return;
      var _props2 = this.props;
      var width = _props2.width;
      var offsetX = _props2.offsetX;
      var offsetY = _props2.offsetY;
      var cols = _props2.cols;
      var margin = _props2.margin;
      var rowHeight = _props2.rowHeight;
      var useCSSTransforms = _props2.useCSSTransforms;
      var draggableCancel = _props2.draggableCancel;
      var draggableHandle = _props2.draggableHandle;

      // Parse 'static'. Any properties defined directly on the grid item will take precedence.

      var draggable = l.static || !this.props.isDraggable ? false : true;
      var resizable = l.static || !this.props.isResizable ? false : true;

      return _react2.default.createElement(
        _GridItem2.default,
        _extends({
          containerWidth: width,
          offsetX: offsetX,
          offsetY: offsetY,
          cols: cols,
          margin: margin,
          rowHeight: rowHeight,
          cancel: draggableCancel,
          handle: draggableHandle,
          onDragStop: this.onDragStop,
          onDragStart: this.onDragStart,
          onDrag: this.onDrag,
          onResizeStart: this.onResizeStart,
          onResize: this.onResize,
          onResizeStop: this.onResizeStop,
          isDraggable: draggable,
          isResizable: resizable,
          useCSSTransforms: useCSSTransforms && this.state.isMounted,
          usePercentages: !this.state.isMounted
        }, l),
        child
      );
    }
  }, {
    key: 'render',
    value: function render() {
      var _this2 = this;

      var _props3 = this.props;
      var className = _props3.className;
      var style = _props3.style;


      var mergedClassName = 'react-grid-layout ' + className;
      var mergedStyle = _extends({
        height: this.containerHeight()
      }, style);

      return _react2.default.createElement(
        'div',
        { className: mergedClassName, style: mergedStyle },
        _react2.default.Children.map(this.props.children, function (child) {
          return _this2.processGridItem(child);
        }),
        this.placeholder()
      );
    }
  }]);

  return ReactGridLayout;
}(_react2.default.Component);

ReactGridLayout.displayName = "ReactGridLayout";
ReactGridLayout.propTypes = {
  //
  // Basic props
  //
  className: _react2.default.PropTypes.string,
  style: _react2.default.PropTypes.object,
  width: _react2.default.PropTypes.number.isRequired,
  offsetY: _react2.default.PropTypes.number.isRequired,
  offsetX: _react2.default.PropTypes.number.isRequired,

  // If true, the container height swells and contracts to fit contents
  autoSize: _react2.default.PropTypes.bool,
  // # of cols.
  cols: _react2.default.PropTypes.number,

  // A selector that will not be draggable.
  draggableCancel: _react2.default.PropTypes.string,
  // A selector for the draggable handler
  draggableHandle: _react2.default.PropTypes.string,

  // If true, the layout will compact vertically
  verticalCompact: _react2.default.PropTypes.bool,

  // layout is an array of object with the format:
  // {x: Number, y: Number, w: Number, h: Number, i: Number}
  layout: function layout(props) {
    var layout = props.layout;
    // I hope you're setting the _grid property on the grid items
    if (layout === undefined) return;
    (0, _utils.validateLayout)(layout, 'layout');
  },

  // margin between items [x, y] in px
  margin: _react2.default.PropTypes.array,
  // Rows have a static height, but you can change this based on breakpoints if you like
  rowHeight: _react2.default.PropTypes.number,

  //
  // Flags
  //
  isDraggable: _react2.default.PropTypes.bool,
  isResizable: _react2.default.PropTypes.bool,
  // Use CSS transforms instead of top/left
  useCSSTransforms: _react2.default.PropTypes.bool,

  //
  // Callbacks
  //

  // Callback so you can save the layout.
  // Calls back with (currentLayout, allLayouts). allLayouts are keyed by breakpoint.
  onLayoutChange: _react2.default.PropTypes.func,

  // Calls when drag starts. Callback is of the signature (layout, oldItem, newItem, placeholder, e).
  // All callbacks below have the same signature. 'start' and 'stop' callbacks omit the 'placeholder'.
  onDragStart: _react2.default.PropTypes.func,
  // Calls on each drag movement.
  onDrag: _react2.default.PropTypes.func,
  // Calls when drag is complete.
  onDragStop: _react2.default.PropTypes.func,
  //Calls when resize starts.
  onResizeStart: _react2.default.PropTypes.func,
  // Calls when resize movement happens.
  onResize: _react2.default.PropTypes.func,
  // Calls when resize is complete.
  onResizeStop: _react2.default.PropTypes.func,

  //
  // Other validations
  //

  // Children must not have duplicate keys.
  children: function children(props, propName, _componentName) {
    _react2.default.PropTypes.node.apply(this, arguments);
    var children = props[propName];

    // Check children keys for duplicates. Throw if found.
    var keys = {};
    _react2.default.Children.forEach(children, function (child) {
      if (keys[child.key]) {
        throw new Error("Duplicate child key found! This will cause problems in ReactGridLayout.");
      }
      keys[child.key] = true;
    });
  }
};
ReactGridLayout.defaultProps = {
  autoSize: true,
  cols: 12,
  rowHeight: 150,
  layout: [],
  margin: [10, 10],
  isDraggable: true,
  isResizable: true,
  useCSSTransforms: true,
  verticalCompact: true,
  onLayoutChange: noop,
  onDragStart: noop,
  onDrag: noop,
  onDragStop: noop,
  onResizeStart: noop,
  onResize: noop,
  onResizeStop: noop
};

var _initialiseProps = function _initialiseProps() {
  this.state = {
    activeDrag: null,
    isMounted: false,
    layout: (0, _utils.synchronizeLayoutWithChildren)(this.props.layout, this.props.children, this.props.cols, this.props.verticalCompact),
    oldDragItem: null,
    oldResizeItem: null
  };
};

exports.default = ReactGridLayout;